==> 20230715.2.zip: misuzu/src/Http/Handlers/HomeHandler.php <==
<?php
namespace Misuzu\Http\Handlers;

use RuntimeException;
use Misuzu\Config;
use Misuzu\Config\IConfig;
use Misuzu\DB;
use Misuzu\Pagination;
use Misuzu\Template;
use Misuzu\Comments\CommentsCategory;
use Misuzu\Users\User;
use Misuzu\Users\UserSession;
use Misuzu\Users\UserNotFoundException;

final class HomeHandler extends Handler {
    public function index($response, $request): void {
        if(UserSession::hasCurrent())
            $this->home($response, $request);
        else
            $this->landing($response, $request);
    }

    public function landing($response, $request): void {
        $linkedData = Config::get('social.embed_linked', IConfig::T_BOOL)
            ? [
                'name' => Config::get('site.name', IConfig::T_STR, 'Misuzu'),
                'url' => Config::get('site.url', IConfig::T_STR),
                'logo' => Config::get('site.ext_logo', IConfig::T_STR),
                'same_as' => Config::get('social.linked', IConfig::T_ARR),
            ] : null;

        $featuredNews = $this->context->getNews()->getAllPosts(
            onlyFeatured: true,
            pagination: new Pagination(3)
        );

        $stats = DB::query(
            'SELECT'
            . ' (SELECT COUNT(`user_id`) FROM `msz_users` WHERE `user_deleted` IS NULL) AS `count_users_all`,'
            . ' (SELECT COUNT(`user_id`) FROM `msz_users` WHERE `user_active` >= DATE_SUB(NOW(), INTERVAL 5 MINUTE)) AS `count_users_online`,'
            . ' (SELECT COUNT(`user_id`) FROM `msz_users` WHERE `user_active` >= DATE_SUB(NOW(), INTERVAL 24 HOUR)) AS `count_users_active`,'
            . ' (SELECT COUNT(`comment_id`) FROM `msz_comments_posts` WHERE `comment_deleted` IS NULL) AS `count_comments`,'
            . ' (SELECT COUNT(`topic_id`) FROM `msz_forum_topics` WHERE `topic_deleted` IS NULL) AS `count_forum_topics`,'
            . ' (SELECT COUNT(`post_id`) FROM `msz_forum_posts` WHERE `post_deleted` IS NULL) AS `count_forum_posts`'
        )->fetch();

        $onlineUsers = DB::query(
            'SELECT u.`user_id`, u.`username`, COALESCE(u.`user_colour`, r.`role_colour`) AS `user_colour`'
            . ' FROM `msz_users` AS u'
            . ' LEFT JOIN `msz_roles` AS r'
            . ' ON r.`role_id` = u.`display_role`'
            . ' WHERE u.`user_active` >= DATE_SUB(NOW(), INTERVAL 5 MINUTE)'
            . ' ORDER BY u.`user_active` DESC, RAND()'
            . ' LIMIT 100'
        )->fetchAll();

        // TODO: don't hardcode forum ids
        $featuredForums = Config::get('landing.forum_categories', IConfig::T_ARR);

        $popularTopics = [];
        $activeTopics = [];

        if(!empty($featuredForums)) {
            $getPopularTopics = DB::prepare(
                'SELECT t.`topic_id`, c.`forum_id`, t.`topic_title`, c.`forum_icon`, t.`topic_count_views`'
                . ', (SELECT COUNT(*) FROM `msz_forum_posts` AS p WHERE p.`topic_id` = t.`topic_id` AND `post_deleted` IS NULL) AS `topic_count_posts`'
                . ' FROM `msz_forum_topics` AS t'
                . ' LEFT JOIN `msz_forum_categories` AS c ON c.`forum_id` = t.`forum_id`'
                . ' WHERE c.`forum_id` IN (' . implode(',', $featuredForums) . ') AND `topic_deleted` IS NULL AND `topic_locked` IS NULL'
                . ' ORDER BY (SELECT COUNT(*) FROM `msz_forum_posts` AS p WHERE p.`topic_id` = t.`topic_id` AND `post_deleted` IS NULL AND `post_created` > NOW() - INTERVAL 3 MONTH) DESC'
            )->stmt;
            $getPopularTopics->execute();
            for($i = 0; $i < 10; ++$i) {
                $topicInfo = $getPopularTopics->fetchObject();
                if(empty($topicInfo))
                    break;
                $popularTopics[] = $topicInfo;
            }

            $getActiveTopics = DB::prepare(
                'SELECT t.`topic_id`, c.`forum_id`, t.`topic_title`, c.`forum_icon`, t.`topic_count_views`'
                . ', (SELECT COUNT(*) FROM `msz_forum_posts` AS p WHERE p.`topic_id` = t.`topic_id` AND `post_deleted` IS NULL) AS `topic_count_posts`'
                . ', (SELECT MAX(`post_id`) FROM `msz_forum_posts` AS p WHERE p.`topic_id` = t.`topic_id` AND `post_deleted` IS NULL) AS `latest_post_id`'
                . ' FROM `msz_forum_topics` AS t'
                . ' LEFT JOIN `msz_forum_categories` AS c ON c.`forum_id` = t.`forum_id`'
                . ' WHERE c.`forum_id` IN (' . implode(',', $featuredForums) . ') AND `topic_deleted` IS NULL AND `topic_locked` IS NULL'
                . ' ORDER BY `topic_bumped` DESC'
            )->stmt;
            $getActiveTopics->execute();
            for($i = 0; $i < 10; ++$i) {
                $topicInfo = $getActiveTopics->fetchObject();
                if(empty($topicInfo))
                    break;
                $activeTopics[] = $topicInfo;
            }
        }

        $response->setContent(Template::renderRaw('home.landing', [
            'statistics' => $stats,
            'online_users' => $onlineUsers,
            'featured_news' => $featuredNews,
            'linked_data' => $linkedData,
            'forum_popular' => $popularTopics,
            'forum_active' => $activeTopics,
        ]));
    }

    public function home($response, $request): void {
        $news = $this->context->getNews();
        $comments = $this->context->getComments();
        $featuredNews = [];
        $userInfos = [];
        $categoryInfos = [];
        $featuredNewsInfos = $news->getAllPosts(
            onlyFeatured: true,
            pagination: new Pagination(5)
        );

        foreach($featuredNewsInfos as $postInfo) {
            $userId = $postInfo->getUserId();
            $categoryId = $postInfo->getCategoryId();

            if(array_key_exists($userId, $userInfos)) {
                $userInfo = $userInfos[$userId];
            } else {
                try {
                    $userInfo = User::byId($userId);
                } catch(UserNotFoundException $ex) {
                    $userInfo = null;
                }

                $userInfos[$userId] = $userInfo;
            }

            if(array_key_exists($categoryId, $categoryInfos))
                $categoryInfo = $categoryInfos[$categoryId];
            else
                $categoryInfos[$categoryId] = $categoryInfo = $news->getCategoryByPost($postInfo);

            $commentsCount = $postInfo->hasCommentsCategoryId()
                ? $comments->countPosts($postInfo->getCommentsCategoryId(), includeReplies: true) : 0;

            $featuredNews[] = [
                'post' => $postInfo,
                'category' => $categoryInfo,
                'user' => $userInfo,
                'comments_count' => $commentsCount,
            ];
        }

        $stats = DB::query(
            'SELECT'
            . ' (SELECT COUNT(`user_id`) FROM `msz_users` WHERE `user_deleted` IS NULL) AS `count_users_all`,'
            . ' (SELECT COUNT(`user_id`) FROM `msz_users` WHERE `user_active` >= DATE_SUB(NOW(), INTERVAL 5 MINUTE)) AS `count_users_online`,'
            . ' (SELECT COUNT(`user_id`) FROM `msz_users` WHERE `user_active` >= DATE_SUB(NOW(), INTERVAL 24 HOUR)) AS `count_users_active`,'
            . ' (SELECT COUNT(`comment_id`) FROM `msz_comments_posts` WHERE `comment_deleted` IS NULL) AS `count_comments`,'
            . ' (SELECT COUNT(`topic_id`) FROM `msz_forum_topics` WHERE `topic_deleted` IS NULL) AS `count_forum_topics`,'
            . ' (SELECT COUNT(`post_id`) FROM `msz_forum_posts` WHERE `post_deleted` IS NULL) AS `count_forum_posts`'
        )->fetch();

        $changelog = $this->context->getChangelog()->getAllChanges(pagination: new Pagination(10));

        $birthdays  = User::byBirthdate();
        $latestUser = !empty($birthdays) ? null : User::byLatest();

        $onlineUsers = DB::query(
            'SELECT u.`user_id`, u.`username`, COALESCE(u.`user_colour`, r.`role_colour`) AS `user_colour`'
            . ' FROM `msz_users` AS u'
            . ' LEFT JOIN `msz_roles` AS r'
            . ' ON r.`role_id` = u.`display_role`'
            . ' WHERE u.`user_active` >= DATE_SUB(NOW(), INTERVAL 5 MINUTE)'
            . ' ORDER BY u.`user_active` DESC, RAND()'
            . ' LIMIT 104'
        )->fetchAll();

        $response->setContent(Template::renderRaw('home.home', [
            'statistics' => $stats,
            'latest_user' => $latestUser,
            'online_users' => $onlineUsers,
            'birthdays' => $birthdays,
            'featured_changelog' => $changelog,
            'featured_news' => $featuredNews,
        ]));
    }
}

==> 20230803.2.zip: misuzu/src/Http/Handlers/HomeHandler.php <==
<?php
namespace Misuzu\Http\Handlers;

use RuntimeException;
use Index\DateTime;
use Misuzu\DB;
use Misuzu\Pagination;
use Misuzu\Template;
use Misuzu\Comments\CommentsCategory;

final class HomeHandler extends Handler {
    private const STATS = [
        'users:active',
        'users:online:recent',
        'users:online:today',
        'comments:posts:visible',
        'forum:topics:visible',
        'forum:posts:visible',
    ];

    public function index($response, $request): void {
        if($this->context->isLoggedIn())
            $this->home($response, $request);
        else
            $this->landing($response, $request);
    }

    public function landing($response, $request): void {
        $users = $this->context->getUsers();
        $config = $this->context->getConfig();
        $counters = $this->context->getCounters();

        if($config->getBoolean('social.embed_linked')) {
            $ldr = $config->getValues([
                ['site.name:s', 'Misuzu'],
                'site.url:s',
                'site.ext_logo:s',
                'social.linked:a'
            ]);
            $linkedData = [
                'name' => $ldr['site.name'],
                'url' => $ldr['site.url'],
                'logo' => $ldr['site.ext_logo'],
                'same_as' => $ldr['social.linked'],
            ];
        } else $linkedData = null;

        $featuredNews = $this->context->getNews()->getAllPosts(
            onlyFeatured: true,
            pagination: new Pagination(3)
        );

        $stats = $counters->get(self::STATS);
        $onlineUserInfos = $users->getUsers(
            lastActiveInMinutes: 5,
            deleted: false,
            orderBy: 'random',
        );

        // can also cheat here, whoa
        $stats['users:online:recent'] = count($onlineUserInfos);

        // TODO: don't hardcode forum ids
        $featuredForums = $config->getArray('landing.forum_categories');

        $popularTopics = [];
        $activeTopics = [];

        if(!empty($featuredForums)) {
            $getPopularTopics = DB::prepare(
                'SELECT t.`topic_id`, c.`forum_id`, t.`topic_title`, c.`forum_icon`, t.`topic_count_views`'
                . ', (SELECT COUNT(*) FROM `msz_forum_posts` AS p WHERE p.`topic_id` = t.`topic_id` AND `post_deleted` IS NULL) AS `topic_count_posts`'
                . ' FROM `msz_forum_topics` AS t'
                . ' LEFT JOIN `msz_forum_categories` AS c ON c.`forum_id` = t.`forum_id`'
                . ' WHERE c.`forum_id` IN (' . implode(',', $featuredForums) . ') AND `topic_deleted` IS NULL AND `topic_locked` IS NULL'
                . ' ORDER BY (SELECT COUNT(*) FROM `msz_forum_posts` AS p WHERE p.`topic_id` = t.`topic_id` AND `post_deleted` IS NULL AND `post_created` > NOW() - INTERVAL 3 MONTH) DESC'
            )->stmt;
            $getPopularTopics->execute();
            for($i = 0; $i < 10; ++$i) {
                $topicInfo = $getPopularTopics->fetchObject();
                if(empty($topicInfo))
                    break;
                $popularTopics[] = $topicInfo;
            }

            $getActiveTopics = DB::prepare(
                'SELECT t.`topic_id`, c.`forum_id`, t.`topic_title`, c.`forum_icon`, t.`topic_count_views`'
                . ', (SELECT COUNT(*) FROM `msz_forum_posts` AS p WHERE p.`topic_id` = t.`topic_id` AND `post_deleted` IS NULL) AS `topic_count_posts`'
                . ', (SELECT MAX(`post_id`) FROM `msz_forum_posts` AS p WHERE p.`topic_id` = t.`topic_id` AND `post_deleted` IS NULL) AS `latest_post_id`'
                . ' FROM `msz_forum_topics` AS t'
                . ' LEFT JOIN `msz_forum_categories` AS c ON c.`forum_id` = t.`forum_id`'
                . ' WHERE c.`forum_id` IN (' . implode(',', $featuredForums) . ') AND `topic_deleted` IS NULL AND `topic_locked` IS NULL'
                . ' ORDER BY `topic_bumped` DESC'
            )->stmt;
            $getActiveTopics->execute();
            for($i = 0; $i < 10; ++$i) {
                $topicInfo = $getActiveTopics->fetchObject();
                if(empty($topicInfo))
                    break;
                $activeTopics[] = $topicInfo;
            }
        }

        $response->setContent(Template::renderRaw('home.landing', [
            'statistics' => $stats,
            'online_users' => $onlineUserInfos,
            'featured_news' => $featuredNews,
            'linked_data' => $linkedData,
            'forum_popular' => $popularTopics,
            'forum_active' => $activeTopics,
        ]));
    }

    public function home($response, $request): void {
        $news = $this->context->getNews();
        $users = $this->context->getUsers();
        $config = $this->context->getConfig();
        $comments = $this->context->getComments();
        $counters = $this->context->getCounters();
        $featuredNews = [];
        $userInfos = [];
        $userColours = [];
        $categoryInfos = [];
        $featuredNewsInfos = $news->getAllPosts(
            onlyFeatured: true,
            pagination: new Pagination(5)
        );

        foreach($featuredNewsInfos as $postInfo) {
            $userId = $postInfo->getUserId();
            $categoryId = $postInfo->getCategoryId();

            if(array_key_exists($userId, $userInfos)) {
                $userInfo = $userInfos[$userId];
                $userColour = $userColours[$userId];
            } else {
                try {
                    $userInfo = $users->getUser($userId, 'id');
                    $userColour = $userColours[$userId] = $users->getUserColour($userInfo);
                } catch(RuntimeException $ex) {
                    $userInfo = null;
                    $userColour = $userColours[$userId] = null;
                }

                $userInfos[$userId] = $userInfo;
            }

            if(array_key_exists($categoryId, $categoryInfos))
                $categoryInfo = $categoryInfos[$categoryId];
            else
                $categoryInfos[$categoryId] = $categoryInfo = $news->getCategoryByPost($postInfo);

            $commentsCount = $postInfo->hasCommentsCategoryId()
                ? $comments->countPosts($postInfo->getCommentsCategoryId(), includeReplies: true) : 0;

            $featuredNews[] = [
                'post' => $postInfo,
                'category' => $categoryInfo,
                'user' => $userInfo,
                'user_colour' => $userColour,
                'comments_count' => $commentsCount,
            ];
        }

        $stats = $counters->get(self::STATS);
        $changelog = $this->context->getChangelog()->getAllChanges(pagination: new Pagination(10));

        $birthdays = [];
        $birthdayInfos = $users->getUsers(deleted: false, birthdate: DateTime::now(), orderBy: 'random');
        foreach($birthdayInfos as $birthdayInfo)
            $birthdays[] = [
                'info' => $birthdayInfo,
                'colour' => $users->getUserColour($birthdayInfo),
            ];

        $newestMember = [];
        if(empty($birthdays)) {
            $newestMemberId = $config->getString('users.newest');
            if(!empty($newestMemberId))
                try {
                    $newestMemberInfo = $users->getUser($newestMemberId, 'id');
                    $newestMember['info'] = $newestMemberInfo;
                    $newestMember['colour'] = $users->getUserColour($newestMemberInfo);
                } catch(RuntimeException $ex) {
                    $newestMember = [];
                    $config->removeValues('users.newest');
                }
        }

        $onlineUserInfos = $users->getUsers(
            lastActiveInMinutes: 5,
            deleted: false,
            orderBy: 'random',
        );

        // today we cheat
        $stats['users:online:recent'] = count($onlineUserInfos);

        $response->setContent(Template::renderRaw('home.home', [
            'statistics' => $stats,
            'newest_member' => $newestMember,
            'online_users' => $onlineUserInfos,
            'birthdays' => $birthdays,
            'featured_changelog' => $changelog,
            'featured_news' => $featuredNews,
        ]));
    }
}
